<?php

namespace src\classes;

class UserSettings
{
    const CONNECTED_OPTION = 'riddle_connected';
    const API_KEY_OPTION = 'riddle_apikey';
    const BEARER_OPTION = 'riddle_bearer';
    const SELECTED_TEAM_OPTION = 'riddle_selected_team';

    private static $key;
    private static $bearer;
    private static $selectedTeam;

    public static function addDefaults() 
    {
        \add_option(self::CONNECTED_OPTION, false);
        \add_option(self::API_KEY_OPTION, '');
        \add_option(self::BEARER_OPTION, '');
        \add_option(self::SELECTED_TEAM_OPTION, '');
    }

    public static function destroySettings() 
    {
        \delete_option(self::CONNECTED_OPTION);
        \delete_option(self::API_KEY_OPTION);
        \delete_option(self::BEARER_OPTION);
        \delete_option(self::SELECTED_TEAM_OPTION);
    }

    public static function loadCredentials() : array
    {

        self::$key = \get_option(self::API_KEY_OPTION, '');
        self::$bearer = \get_option(self::BEARER_OPTION, '');

        return [
            "apikey" => self::$key,
            "bearer" => self::$bearer,
        ];
    }

    public static function saveCredentials(string $key, string $bearer) 
    {
        \update_option(self::API_KEY_OPTION, $key);
        \update_option(self::BEARER_OPTION, $bearer);
        self::setConnected(true);
    }

    public static function setSelectedTeam(int $teamId)
    {
        self::$selectedTeam = $teamId;
        \update_option(self::SELECTED_TEAM_OPTION, $teamId);
    }

    public static function getSelectedTeam(): int
    {
        if (self::$selectedTeam) {
            return self::$selectedTeam;
        }

        return \get_option(self::SELECTED_TEAM_OPTION, -1);
    }

    public static function usesNewApi(): bool
    {
        if (!self::$key) {
            self::loadCredentials();
        }

        return self::$bearer !== '' && self::$key !== '';
    }

    public static function disconnect()
    {
        \update_option(self::CONNECTED_OPTION, false);
    }

    public static function isConnected(): bool
    {
        $optionValue = \get_option(self::CONNECTED_OPTION, false);

        if (null === $optionValue && '' !== self::loadCredentials()['bearer']) { // if the option is not set but the credentials are there
            self::setConnected(true);

            return true;
        }

        return boolval($optionValue);
    }

    public static function setConnected(bool $connected)
    {
        \update_option(self::CONNECTED_OPTION, $connected);
    }
}