<?php

namespace src\controller;

use src\classes\Controller;
use src\classes\UserSettings;
use src\classes\Landingpage\RiddleLandingpageManager;

class RiddleAdminController extends Controller
{   
    public static function riddleList() 
    {
        if (isset($_GET['subpage'])) {
            $subpage = $_GET['subpage'];

            if ('leads' === $subpage) {
                return RiddleCRPController::riddleLeaderboardEntries();
            }

            return RiddleCRPController::riddleCustomResultPages();
        }

        global $riddleLoader, $riddleApiKey;
        
        if (!$riddleApiKey) { // if the user hasn't put in a key yet
            return self::view("error-nokey-defined.php");
        }

        $team = self::_updateSelectedTeam($riddleLoader);
        $riddles = $riddleLoader->getRiddles();
        $teams = $riddleLoader->getTeamlist();

        if (!is_array($riddles)) {
            return self::view("error-invalidtoken.php");
        }

        return self::view('pages/my-riddles.php', [
            'teams' => $teams,
            'riddles' => $riddles,
            'selectedTeam' => $team,
            'landingPages' => RiddleLandingpageManager::getLandingpagesRiddleIdArray(),
        ]);
    }

    public static function riddleUserSettings() 
    {
        $credentials = self::_getAPICredentials();
        
        return self::view('pages/connect-account.php', $credentials);
    }

    public static function riddleProcessSettings() 
    {
        $key = self::_getPOSTValue('apikey');
        $bearer = self::_getPOSTValue('bearer');

        if ($key && $bearer) { // if the form has been submitted
            UserSettings::destroySettings();
            UserSettings::saveCredentials($key, $bearer);

            return self::redirectToAdminpage("riddle-admin-menu"); // Redirect to "My Riddles" page after connecting
        }
    }

    public static function riddleProcessDisconnect()
    {
        $disconnect = boolval(self::_getGETValue('disconnect'));

        if ($disconnect) {
            UserSettings::disconnect();

            return self::redirectToAdminpage("riddle-admin-menu"); // Redirect to "Connect Account" page
        }
    }

    public static function riddleHelp() 
    {
        return self::view('pages/help.php');
    }

    private static function _updateSelectedTeam(&$riddleLoader)
    {
        $teamId = self::_getPOSTValue('team', false);

        if ($teamId || $teamId === '') {
            UserSettings::setSelectedTeam($teamId);
        }

        $teamId = UserSettings::getSelectedTeam();

        if ($teamId) {
            $riddleLoader->setActiveTeam($teamId);

            return $teamId;
        }

        return false;
    }

    private static function _getAPICredentials()
    {
        return UserSettings::loadCredentials();
    }

}